# Note: this file has been initially generated by Module::Build::Convert 0.49

use Module::Build;

my $build = Module::Build->new
   (
    dist_author => 'Mathieu Jondet <mathieu@eulerian.com>, Xavier Thorillon <xavier@eulerian.com>',
    module_name => 'API::Eulerian',
    dist_abstract => 'Simple interface for all the API provided by Eulerian Technologies.',
    create_readme => 1,
    dist_version_from  => 'lib/API/Eulerian.pm',
    dist_name => 'API-Eulerian',
    create_makefile_pl => 'traditional',
    license => 'perl',
    requires => {
                  'Encode' => 0,
                  'File::Slurp' => 0,
                  'JSON' => 0,
                  'JSON::Streaming::Reader' => 0,
                  'LWP::UserAgent' => 0,
                  'Protocol::WebSocket::Client' => 0,
                  'Switch' => 0,
                  'Text::CSV' => 0,
                  'Time::HiRes' => 0,
                }
   );

$build->create_build_script;
